package cn.com.qmth.examcloud.exchange.inner.service.impl;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.dysmsapi.model.v20170525.QuerySendDetailsRequest;
import com.aliyuncs.dysmsapi.model.v20170525.QuerySendDetailsResponse;
import com.aliyuncs.dysmsapi.model.v20170525.SendSmsRequest;
import com.aliyuncs.dysmsapi.model.v20170525.SendSmsResponse;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.exceptions.ServerException;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.profile.IClientProfile;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.redis.RedisClient;
import cn.com.qmth.examcloud.exchange.inner.service.SendSmsService;
import cn.com.qmth.examcloud.exchange.inner.service.bean.CaptureFailedAlarmInfo;
import cn.com.qmth.examcloud.exchange.inner.service.bean.SendSmsInfo;
import cn.com.qmth.examcloud.exchange.inner.service.bean.SmsCodeRedisInfo;

@Service("sendSmsService")
public class SendSmsServiceImpl implements SendSmsService {
	
	@Autowired
	private RedisClient redisClient;

	//redis中短信验证码key的前缀
	private static final String CODE_REDIS_KEY_PREFIXS = "SMSCODE_";
	
	//验证码默认有效时间
	private static final int DEFAULT_DFFECTIVETIME = 120;
	
	//短信发送默认间隔时间
	private static final int DEFAULT_INTERVALSECONDS = 90;
	
	// 产品名称:云通信短信API产品,开发者无需替换
	private static final String product = "Dysmsapi";

	// 产品域名,开发者无需替换
	private static final String domain = "dysmsapi.aliyuncs.com";

	@Value("${aliyun.sms.accessKeyId}")
	private String accessKeyId;
	
	@Value("${aliyun.sms.accessKeySecret}")
	private String accessKeySecret;
	
	@Override
	public void sendIdentifyingCode(SendSmsInfo sendSmsInfo) {
		if(sendSmsInfo.getEffectiveTime() == null || sendSmsInfo.getEffectiveTime() == 0){
			sendSmsInfo.setEffectiveTime(DEFAULT_DFFECTIVETIME);
		}
		if(sendSmsInfo.getIntervalSeconds() == null|| sendSmsInfo.getIntervalSeconds() == 0){
			sendSmsInfo.setIntervalSeconds(DEFAULT_INTERVALSECONDS);
		}
		SmsCodeRedisInfo smsCodeRedisInfo = redisClient.get(CODE_REDIS_KEY_PREFIXS+sendSmsInfo.getPhone(),SmsCodeRedisInfo.class);
		if(smsCodeRedisInfo != null){
			long createTime = smsCodeRedisInfo.getCreateTime();
			if((System.currentTimeMillis()-createTime)<sendSmsInfo.getIntervalSeconds()*60*1000){
				throw new StatusException("EX-sendIdentifyingCode-1","同一个手机号码,"+sendSmsInfo.getIntervalSeconds()+"秒内允许请求一次");
			}
		}
		
		// 可自助调整超时时间
		System.setProperty("sun.net.client.defaultConnectTimeout", "10000");
		System.setProperty("sun.net.client.defaultReadTimeout", "10000");

		// 初始化acsClient,暂不支持region化
		IClientProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKeyId,accessKeySecret);
		try {
			DefaultProfile.addEndpoint("cn-hangzhou", "cn-hangzhou", product, domain);
		} catch (ClientException e) {
			throw new StatusException("EX-sendIdentifyingCode-1", "DefaultProfile.addEndpoint failed");
		}
		IAcsClient acsClient = new DefaultAcsClient(profile);

		// 组装请求对象-具体描述见控制台-文档部分内容
		SendSmsRequest request = new SendSmsRequest();
		// 必填:待发送手机号
		request.setPhoneNumbers(sendSmsInfo.getPhone());
		// 必填:短信签名-可在短信控制台中找到
		request.setSignName(sendSmsInfo.getSign());
		// 必填:短信模板-可在短信控制台中找到
		request.setTemplateCode(sendSmsInfo.getTemplateCode());
		// 可选:模板中的变量替换JSON串,如模板内容为"亲爱的${name},您的验证码为${code}"时,此处的值为
		
		request.setTemplateParam("{\"code\":\""+sendSmsInfo.getCode()+"\"}");

		// 选填-上行短信扩展码(无特殊需求用户请忽略此字段)
		// request.setSmsUpExtendCode("90997");

		// 可选:outId为提供给业务方扩展字段,最终在短信回执消息中将此值带回给调用者
		request.setOutId("yourOutId");

		// hint 此处可能会抛出异常，注意catch
		try {
			SendSmsResponse sendSmsResponse = acsClient.getAcsResponse(request);
			if("ok".equalsIgnoreCase(sendSmsResponse.getCode())){
				setCodeInfoToRedis(sendSmsInfo);
			}else{
				throw new StatusException("EX-sendIdentifyingCode-2", "code:"+sendSmsResponse.getCode()+",message:"+sendSmsResponse.getMessage());
			}
		} catch (ServerException e) {
			throw new StatusException("EX-sendIdentifyingCode-3", "send sms failed");
		} catch (ClientException e) {
			throw new StatusException("EX-sendIdentifyingCode-3", "send sms failed");
		}
	}
	
	/**
	 * 将短信验证码信息放入redis
	 * @param sendSmsInfo
	 */
	private void setCodeInfoToRedis(SendSmsInfo sendSmsInfo){
		String redisKey = CODE_REDIS_KEY_PREFIXS+sendSmsInfo.getPhone();
		long currentTime = System.currentTimeMillis();
		
		SmsCodeRedisInfo smsCodeRedisInfo = new SmsCodeRedisInfo();
		smsCodeRedisInfo.setCode(sendSmsInfo.getCode());
		smsCodeRedisInfo.setCreateTime(currentTime);
		redisClient.set(redisKey, smsCodeRedisInfo, sendSmsInfo.getEffectiveTime());
	}
	
	@Override
	public QuerySendDetailsResponse querySendDetails(String bizId,String phone){

		// 可自助调整超时时间
		System.setProperty("sun.net.client.defaultConnectTimeout", "10000");
		System.setProperty("sun.net.client.defaultReadTimeout", "10000");

		// 初始化acsClient,暂不支持region化
		IClientProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKeyId,
				accessKeySecret);
		try {
			DefaultProfile.addEndpoint("cn-hangzhou", "cn-hangzhou", product, domain);
		} catch (ClientException e) {
			e.printStackTrace();
		}
		IAcsClient acsClient = new DefaultAcsClient(profile);

		// 组装请求对象
		QuerySendDetailsRequest request = new QuerySendDetailsRequest();
		// 必填-号码
		request.setPhoneNumber(phone);
		// 可选-流水号
		request.setBizId(bizId);
		// 必填-发送日期 支持30天内记录查询，格式yyyyMMdd
		SimpleDateFormat ft = new SimpleDateFormat("yyyyMMdd");
		request.setSendDate(ft.format(new Date()));
		// 必填-页大小
		request.setPageSize(10L);
		// 必填-当前页码从1开始计数
		request.setCurrentPage(1L);

		// hint 此处可能会抛出异常，注意catch
		QuerySendDetailsResponse querySendDetailsResponse = null;
		try {
			querySendDetailsResponse = acsClient.getAcsResponse(request);
		} catch (ServerException e) {
			e.printStackTrace();
		} catch (ClientException e) {
			e.printStackTrace();
		}

		return querySendDetailsResponse;
	}

	@Override
	public boolean checkIdentifyingCode(String phone, String code) {
		SmsCodeRedisInfo smsCodeRedisInfo = redisClient.get(CODE_REDIS_KEY_PREFIXS+phone, SmsCodeRedisInfo.class);
		if(smsCodeRedisInfo!=null){
			if(code.equals(smsCodeRedisInfo.getCode()+"")){
				//校验成功,删除验证码
				redisClient.delete(CODE_REDIS_KEY_PREFIXS+phone);
				return true;
			}
		}
		return false;
	}

	@Override
	public void captureFailedAlarm(CaptureFailedAlarmInfo captureFailedAlarmInfo) throws Exception {
		// 可自助调整超时时间
		System.setProperty("sun.net.client.defaultConnectTimeout", "10000");
		System.setProperty("sun.net.client.defaultReadTimeout", "10000");

		// 初始化acsClient,暂不支持region化
		IClientProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKeyId,accessKeySecret);
		DefaultProfile.addEndpoint("cn-hangzhou", "cn-hangzhou", product, domain);
		IAcsClient acsClient = new DefaultAcsClient(profile);

		// 组装请求对象-具体描述见控制台-文档部分内容
		SendSmsRequest request = new SendSmsRequest();
		// 必填:待发送手机号
		request.setPhoneNumbers(captureFailedAlarmInfo.getPhone());
		// 必填:短信签名-可在短信控制台中找到
		request.setSignName(captureFailedAlarmInfo.getSign());
		// 必填:短信模板-可在短信控制台中找到
		request.setTemplateCode(captureFailedAlarmInfo.getTemplateCode());
		// 可选:模板中的变量
		request.setTemplateParam(captureFailedAlarmInfo.getTemplateParam());
		// 可选:outId为提供给业务方扩展字段,最终在短信回执消息中将此值带回给调用者
		request.setOutId("yourOutId");

		// hint 此处可能会抛出异常，注意catch
		acsClient.getAcsResponse(request);
	}
	
}
