package cn.com.qmth.examcloud.exchange.outer.api.controller;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;

import org.apache.commons.fileupload.disk.DiskFileItem;
import org.apache.tomcat.util.http.fileupload.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.ZipUtil;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.exchange.outer.service.FaceService;
import io.swagger.annotations.ApiOperation;

/**
 * 人脸 处理
 * 
 * @author chenken
 * @date 2018年4月3日 下午3:58:31
 * @company QMTH
 * @description FaceppController.java
 */
@RestController
@RequestMapping("/api/ecs_outlet/facepp")
public class FaceController extends ControllerSupport {

	@Value("${$tempDir}")
	private String tempDir;

	/**
	 * ZIP最大50M
	 */
	private static final int ZIP_MAX_SIZE = 50;

	/**
	 * 每张照片最大尺寸 500KB
	 */
	private static final int MAX_SIZE = 500;

	@Autowired
	private FaceService faceService;

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param file
	 */
	@ApiOperation(value = "保存学生照片")
	@PostMapping("add")
	public void addPhoto(@RequestParam CommonsMultipartFile file) {
		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		log.debug("fileName =" + file.getOriginalFilename());
		DiskFileItem item = (DiskFileItem) file.getFileItem();
		File storeLocation = item.getStoreLocation();
		String fileName = file.getOriginalFilename();

		if (StringUtils.containsWhitespace(fileName)) {
			throw new StatusException("EX-600100", "文件名不能含有空白字符");
		}

		if (!fileName.matches("[^\\.\\s]+\\.[^\\.\\s]+")) {
			throw new StatusException("EX-600101", "文件名不合法");
		}
		String identityNumber = fileName.substring(0, fileName.lastIndexOf("."));
		String fileSuffix = fileName.substring(fileName.lastIndexOf("."));

		faceService.processFace(rootOrgId, identityNumber, fileSuffix, storeLocation,
				accessUser.getDisplayName());
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param file
	 * @return
	 */
	@ApiOperation(value = "导入学生照片")
	@PostMapping("import")
	public List<Map<String, String>> importPhotos(@RequestParam CommonsMultipartFile file) {

		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		DiskFileItem item = (DiskFileItem) file.getFileItem();
		File storeLocation = item.getStoreLocation();
		String filename = file.getOriginalFilename();

		if (!filename.endsWith(".zip")) {
			throw new StatusException("EX-620001", "文件格式不正确，必须是zip格式的压缩包");
		}
		// 文件大小限制
		if (file.getSize() > ZIP_MAX_SIZE * 1024 * 1024) {
			throw new StatusException("EX-620002", "文件大小超过50M,请分批导入");
		}

		List<Map<String, String>> ret = Lists.newArrayList();

		String tempDirPath = tempDir + File.separator + "photo_import" + File.separator
				+ System.currentTimeMillis();
		File tempDir = new File(tempDirPath);
		try {
			ZipUtil.unzip(storeLocation, new File(tempDirPath));
		} catch (Exception e) {
			throw new StatusException("EX-620003", "zip文件损坏");
		}

		File[] files = tempDir.listFiles();

		if (null == files) {
			throw new StatusException("EX-620004", "zip文件为空");
		}

		for (File f : files) {
			Map<String, String> map = Maps.newHashMap();
			String fileName = f.getName();
			map.put("file", fileName);

			if (f.length() > MAX_SIZE * 1024) {
				map.put("statusCode", "EX-620005");
				map.put("statusDesc", "文件大小超过500KB");
				ret.add(map);
				continue;
			}
			try {
				if (StringUtils.containsWhitespace(fileName)) {
					throw new StatusException("EX-600100", "文件名不能含有空白字符");
				}

				if (!fileName.matches("[^\\.\\s]+\\.[^\\.\\s]+")) {
					throw new StatusException("EX-600101", "文件名不合法");
				}
				String identityNumber = fileName.substring(0, fileName.lastIndexOf("."));
				String fileSuffix = fileName.substring(fileName.lastIndexOf("."));

				faceService.processFace(rootOrgId, identityNumber, fileSuffix, f,
						accessUser.getDisplayName());

				map.put("statusCode", "200");
				map.put("statusDesc", "成功");

			} catch (StatusException e) {
				map.put("statusCode", e.getCode());
				map.put("statusDesc", e.getDesc());
			} catch (Exception e) {
				map.put("statusCode", "系统异常");
				map.put("statusDesc", e.getCause().getMessage());
			}
			ret.add(map);
		}

		try {
			FileUtils.deleteDirectory(tempDir);
		} catch (IOException e) {
			log.error("fail to clean temp dir.", e);
		}

		return ret;
	}

}
