package cn.com.qmth.examcloud.exchange.outer.api.controller.swufe;

import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.SaveStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.SaveStudentResp;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.ExamStudentCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;
import cn.com.qmth.examcloud.examwork.api.bean.ExamStudentBean;
import cn.com.qmth.examcloud.examwork.api.request.GetExamReq;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamStudentReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamResp;
import cn.com.qmth.examcloud.examwork.api.response.SaveExamStudentResp;
import cn.com.qmth.examcloud.exchange.base.enums.CourseLevel;
import cn.com.qmth.examcloud.exchange.outer.api.controller.swufe.bean.StudentInfoReq;

/**
 * @author chenken
 * @date 2018年5月7日 下午4:30:29
 * @company QMTH
 * @description 保存学生信息
 */
@RestController
@RequestMapping("/api/ecs_outlet/studentInfo")
public class StudentInfoController extends ControllerSupport {

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	ExamCloudService examCloudService;

	@Autowired
	ExamStudentCloudService examStudentCloudService;

	@PostMapping
	public Map<String, Object> saveStudentInfo(@RequestBody List<StudentInfoReq> studentInfoReqs) {
		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		List<Long> examStudentIdList = Lists.newArrayList();
		for (StudentInfoReq cur : studentInfoReqs) {

			GetExamReq req = new GetExamReq();
			req.setName(cur.getExamName());
			req.setRootOrgId(rootOrgId);
			GetExamResp resp = examCloudService.getExam(req);

			Long examId = resp.getId();
			ExamBean examBean = resp.getExamBean();
			String examName = examBean.getName();

			SaveStudentReq saveStudentReq = new SaveStudentReq();
			saveStudentReq.setIdentityNumber(cur.getIdentityNumber());
			saveStudentReq.setName(cur.getStudentName());
			saveStudentReq.setOrgCode(cur.getOrgCode());
			saveStudentReq.setOrgId(cur.getOrgId());
			saveStudentReq.setOrgName(cur.getOrgName());
			saveStudentReq.setRootOrgId(rootOrgId);
			saveStudentReq.setStudentCode(cur.getStudentCode());

			SaveStudentResp insertOrUpdateStudentResp = studentCloudService
					.saveStudent(saveStudentReq);

			Long studentId = insertOrUpdateStudentResp.getStudentId();

			SaveExamStudentReq sReq = new SaveExamStudentReq();
			sReq.setCourseLevel(cur.getCourseLevel());

			CourseLevel courseLevel = CourseLevel.getCourseLevel(cur.getCourseLevel());
			String courseCode = courseLevel.getAbbreviate() + cur.getCourseCode();
			sReq.setCourseCode(courseCode);

			sReq.setCourseName(cur.getCourseName());
			sReq.setExamId(examId);
			sReq.setStudentId(studentId);
			sReq.setExamName(examName);
			sReq.setIdentityNumber(cur.getIdentityNumber());
			if (StringUtils.isNotBlank(cur.getPaperType())) {
				sReq.setPaperType(cur.getPaperType());
			} else {
				sReq.setPaperType("O");
			}
			sReq.setRootOrgId(rootOrgId);
			sReq.setStudentCode(cur.getStudentCode());
			sReq.setStudentName(cur.getStudentName());
			sReq.setRemark(cur.getRemark());
			SaveExamStudentResp savedExamStudent = examStudentCloudService.saveExamStudent(sReq);
			ExamStudentBean examStudentBean = savedExamStudent.getExamStudentBean();
			examStudentIdList.add(examStudentBean.getId());
		}

		Map<String, Object> map = Maps.newHashMap();
		map.put("success", true);
		map.put("examStudentIdList", examStudentIdList);
		return map;
	}

}
