package cn.com.qmth.examcloud.exchange.outer.api.controller.sydx;

import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.DateUtil;
import cn.com.qmth.examcloud.commons.base.util.DateUtil.DatePatterns;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.SaveStudentReq;
import cn.com.qmth.examcloud.core.basic.api.request.UnbindStudentCodeReq;
import cn.com.qmth.examcloud.core.basic.api.response.SaveStudentResp;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.ExamStudentCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;
import cn.com.qmth.examcloud.examwork.api.bean.ExamStudentBean;
import cn.com.qmth.examcloud.examwork.api.request.GetExamReq;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamReq;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamStudentReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamResp;
import cn.com.qmth.examcloud.examwork.api.response.SaveExamStudentResp;
import cn.com.qmth.examcloud.exchange.base.enums.CourseLevel;
import cn.com.qmth.examcloud.exchange.outer.api.controller.sydx.bean.ExamStudentImportRequest;
import cn.com.qmth.examcloud.exchange.outer.api.controller.sydx.bean.FailureRecordDomain;
import cn.com.qmth.examcloud.exchange.outer.api.controller.sydx.bean.OutletExam;
import cn.com.qmth.examcloud.exchange.outer.api.controller.sydx.bean.OutletExamStudent;

/**
 * 石油大学专用-接收考生数据接口
 * 
 * @author chenken
 * @date 2018年4月3日 下午3:57:42
 * @company QMTH
 * @description StudentExamInfoController.java
 */
@RestController
@RequestMapping("/api/ecs_outlet/sydx/import_exam_student_info")
public class StudentExamInfoController extends ControllerSupport {

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	ExamCloudService examCloudService;

	@Autowired
	ExamStudentCloudService examStudentCloudService;

	/**
	 * 石油大学使用:创建考试和导入学生
	 *
	 * @author WANGWEI
	 * @param outerReq
	 * @return
	 */
	@PostMapping
	public ResponseEntity<?> post(@RequestBody ExamStudentImportRequest outerReq) {

		tranTimes(outerReq);

		OutletExam exam = outerReq.getExam();

		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		GetExamReq req = new GetExamReq();
		req.setName(exam.getName());
		req.setRootOrgId(accessUser.getRootOrgId());
		GetExamResp resp = null;
		try {
			resp = examCloudService.getExam(req);
		} catch (StatusException e) {
			if (e.getCode().equals("E-002002")) {
				SaveExamReq saveExamReq = new SaveExamReq();
				saveExamReq.setBeginTime(DateUtil.parse(exam.getBeginTime(), DatePatterns.ISO));
				saveExamReq.setDuration(exam.getDuration());
				saveExamReq.setEndTime(DateUtil.parse(exam.getEndTime(), DatePatterns.ISO));
				saveExamReq.setExamType(exam.getExamType().name());
				saveExamReq.setName(exam.getName());
				saveExamReq.setRemark(exam.getRemark());
				saveExamReq.setRootOrgId(rootOrgId);
				examCloudService.saveExam(saveExamReq);
				resp = examCloudService.getExam(req);
			} else {
				throw e;
			}
		}

		Long examId = resp.getId();
		ExamBean examBean = resp.getExamBean();
		String examName = examBean.getName();
		List<OutletExamStudent> examStudentList = outerReq.getExamStudentList();

		List<Long> examStudentIdList = Lists.newArrayList();
		List<FailureRecordDomain> failureRecordList = Lists.newArrayList();

		for (OutletExamStudent cur : examStudentList) {

			SaveStudentReq saveStudentReq = new SaveStudentReq();
			saveStudentReq.setIdentityNumber(cur.getIdentityNumber());
			saveStudentReq.setName(cur.getName());
			saveStudentReq.setPhoneNumber(cur.getPhone());
			saveStudentReq.setOrgCode(cur.getOrgCode());
			saveStudentReq.setOrgId(cur.getOrgId());
			saveStudentReq.setOrgName(cur.getOrgName());
			saveStudentReq.setRootOrgId(rootOrgId);
			saveStudentReq.setStudentCode(cur.getStudentCode());

			SaveStudentResp saveStudentResp = null;
			try {
				saveStudentResp = studentCloudService.saveStudent(saveStudentReq);
			} catch (StatusException e) {
				String code = e.getCode();
				if (code.equals("B-160005") || code.equals("B-160008")) {
					UnbindStudentCodeReq unbindStudentCodeReq = new UnbindStudentCodeReq();
					unbindStudentCodeReq.setRootOrgId(rootOrgId);
					unbindStudentCodeReq.setStudentCode(cur.getStudentCode());
					unbindStudentCodeReq.setIdentityNumber(cur.getIdentityNumber());
					studentCloudService.unbindStudentCode(unbindStudentCodeReq);
					saveStudentResp = studentCloudService.saveStudent(saveStudentReq);
				} else {
					throw e;
				}
			}

			Long studentId = saveStudentResp.getStudentId();

			SaveExamStudentReq sReq = new SaveExamStudentReq();
			sReq.setCourseLevel(cur.getCourseLevel());

			CourseLevel courseLevel = CourseLevel.getCourseLevel(cur.getCourseLevel());
			String courseCode = courseLevel.getAbbreviate() + cur.getCourseCode();
			sReq.setCourseCode(courseCode);

			sReq.setCourseName(cur.getCourseName());
			sReq.setExamId(examId);
			sReq.setStudentId(studentId);
			sReq.setExamName(examName);
			sReq.setIdentityNumber(cur.getIdentityNumber());
			sReq.setPaperType(cur.getPaperType());
			sReq.setRootOrgId(rootOrgId);
			sReq.setStudentCode(cur.getStudentCode());
			sReq.setStudentName(cur.getName());
			sReq.setPaperType(cur.getPaperType());
			sReq.setRemark(cur.getRemark());
			sReq.setGrade(cur.getGrade());
			sReq.setInfoCollector(cur.getInfoCollector());
			sReq.setSpecialtyName(cur.getSpecialtyName());

			try {
				SaveExamStudentResp savedExamStudent = examStudentCloudService
						.saveExamStudent(sReq);
				ExamStudentBean examStudentBean = savedExamStudent.getExamStudentBean();
				examStudentIdList.add(examStudentBean.getId());
			} catch (StatusException e) {
				FailureRecordDomain fr = new FailureRecordDomain();
				fr.setCode(e.getCode());
				fr.setDesc(e.getDesc());
				fr.setCourseCode(cur.getCourseCode());
				fr.setCourseLevel(cur.getCourseLevel());
				fr.setCourseName(cur.getCourseName());
				fr.setIdentityNumber(cur.getIdentityNumber());
				fr.setStudentCode(cur.getStudentCode());
				failureRecordList.add(fr);
			} catch (Exception e) {
				FailureRecordDomain fr = new FailureRecordDomain();
				fr.setCode("EX-210001");
				fr.setDesc(e.getMessage());
				fr.setCourseCode(cur.getCourseCode());
				fr.setCourseLevel(cur.getCourseLevel());
				fr.setCourseName(cur.getCourseName());
				fr.setIdentityNumber(cur.getIdentityNumber());
				fr.setStudentCode(cur.getStudentCode());
				failureRecordList.add(fr);
			}
		}

		if (CollectionUtils.isNotEmpty(failureRecordList)) {
			return new ResponseEntity<Object>(failureRecordList, HttpStatus.INTERNAL_SERVER_ERROR);
		} else {
			return new ResponseEntity<Object>(examStudentIdList, HttpStatus.OK);
		}
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examStudentImportDTO
	 */
	private void tranTimes(ExamStudentImportRequest examStudentImportDTO) {
		OutletExam exam = examStudentImportDTO.getExam();

		String beginTime = exam.getBeginTime();
		if (StringUtils.isNotBlank(beginTime)) {
			beginTime = StringUtils.replace(beginTime, ".000Z", "");
			beginTime = StringUtils.replace(beginTime, "T", " ");
			exam.setBeginTime(beginTime);
		}

		String endTime = exam.getEndTime();
		if (StringUtils.isNotBlank(endTime)) {
			endTime = StringUtils.replace(endTime, ".000Z", "");
			endTime = StringUtils.replace(endTime, "T", " ");
			exam.setEndTime(endTime);
		}

		List<OutletExamStudent> examStudentList = examStudentImportDTO.getExamStudentList();
		if (CollectionUtils.isNotEmpty(examStudentList)) {

			for (OutletExamStudent cur : examStudentList) {
				OutletExam curExam = cur.getExam();
				if (null == curExam) {
					continue;
				}
				String b = curExam.getBeginTime();
				if (StringUtils.isNotBlank(b)) {
					b = StringUtils.replace(b, ".000Z", "");
					b = StringUtils.replace(b, "T", " ");
					curExam.setBeginTime(b);
				}

				String e = curExam.getEndTime();
				if (StringUtils.isNotBlank(e)) {
					e = StringUtils.replace(e, ".000Z", "");
					e = StringUtils.replace(e, "T", " ");
					curExam.setEndTime(e);
				}
			}
		}
	}

}
