package cn.com.qmth.examcloud.exchange.outer.api.provider;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.commons.web.support.StatusResponse;
import cn.com.qmth.examcloud.examwork.api.ExamStudentCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamStudentBean;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamStudentReq;
import cn.com.qmth.examcloud.examwork.api.response.SaveExamStudentResp;
import cn.com.qmth.examcloud.exchange.outer.api.ExamStudentOuterService;
import cn.com.qmth.examcloud.exchange.outer.api.bean.OuterExamStudentBean;
import cn.com.qmth.examcloud.exchange.outer.api.request.OuterSaveExamStudentReq;
import cn.com.qmth.examcloud.exchange.outer.api.response.OuterSaveExamStudentResp;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

/**
 * 考生服务
 *
 * @author WANGWEI
 * @date 2018年7月2日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Api(value = "考生服务")
@RestController
@RequestMapping("${$rmp.cloud.exchange.outer}/examStudent")
public class ExamStudentOuterServiceProvider extends ControllerSupport
		implements
			ExamStudentOuterService {

	private static final long serialVersionUID = -8252740695302763716L;

	@Autowired
	ExamStudentCloudService examStudentCloudService;

	@ApiOperation(value = "保存考生信息", httpMethod = "POST")
	@ApiResponses({
			@ApiResponse(code = 200, message = "成功", response = OuterSaveExamStudentResp.class),
			@ApiResponse(code = 500, message = "系统异常(异常信息见响应体)", response = StatusResponse.class)})
	@PostMapping("saveExamStudent")
	@Override
	public OuterSaveExamStudentResp saveExamStudent(
			@RequestBody @ApiParam(required = true) OuterSaveExamStudentReq req) {

		if (!getSecurityRootOrgId().equals(req.getRootOrgId())) {
			throw new StatusException("EX-1000001", "rootOrgId is wrong");
		}

		SaveExamStudentReq request = new SaveExamStudentReq();

		request.setCourseCode(req.getCourseCode());
		request.setCourseLevel(req.getCourseLevel());
		request.setCourseName(req.getCourseName());
		request.setExamId(req.getExamId());
		request.setExamName(req.getExamName());
		request.setIdentityNumber(req.getIdentityNumber());
		request.setPaperType(req.getPaperType());
		request.setRootOrgId(req.getRootOrgId());
		request.setStudentCode(req.getStudentCode());
		request.setStudentName(req.getStudentName());
		request.setInfoCollector(req.getInfoCollector());
		request.setSpecialtyName(req.getSpecialtyName());
		request.setExamSite(req.getExamSite());
		request.setGrade(req.getGrade());
		request.setRemark(req.getRemark());

		SaveExamStudentResp response = examStudentCloudService.saveExamStudent(request);

		ExamStudentBean sb = response.getExamStudentBean();

		OuterSaveExamStudentResp resp = new OuterSaveExamStudentResp();
		OuterExamStudentBean bean = resp.getExamStudentBean();
		bean.setCourseCode(sb.getCourseCode());
		bean.setCourseLevel(sb.getCourseLevel());
		bean.setCourseName(sb.getCourseName());
		bean.setExamId(sb.getExamId());
		bean.setExamName(sb.getExamName());
		bean.setIdentityNumber(sb.getIdentityNumber());
		bean.setPaperType(sb.getPaperType());
		bean.setRootOrgId(sb.getRootOrgId());
		bean.setStudentCode(sb.getStudentCode());
		bean.setStudentName(sb.getStudentName());

		resp.setExamStudentBean(bean);

		return resp;
	}

}
