package cn.com.qmth.examcloud.exchange.outer.api.provider;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;

import org.apache.commons.fileupload.disk.DiskFileItem;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.HttpClientUtil;
import cn.com.qmth.examcloud.commons.base.util.ZipUtil;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.exchange.outer.api.FaceOuterService;
import cn.com.qmth.examcloud.exchange.outer.service.FaceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * 人脸服务
 *
 * @author WANGWEI
 * @date 2018年11月14日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Api("人脸服务")
@RestController
@RequestMapping("${$rmp.cloud.exchange.outer}/face")
public class FaceOuterServiceProvider extends ControllerSupport implements FaceOuterService {

	private static final long serialVersionUID = -6268268043341270752L;

	@Value("${$tempDir}")
	private String tempDir;

	/**
	 * ZIP最大50M
	 */
	private static final int ZIP_MAX_SIZE = 50;

	/**
	 * 每张照片最大尺寸 500KB
	 */
	private static final int MAX_SIZE = 500;

	@Autowired
	private FaceService faceService;

	@ApiOperation(value = "保存学生照片", httpMethod = "POST")
	@PostMapping("addPhotoByUrl")
	@Override
	public void addPhotoByUrl(
			@RequestParam @ApiParam(value = "顶级机构ID", example = "1", required = true) Long rootOrgId,
			@RequestParam @ApiParam(value = "身份证", required = true) String identityNumber,
			@RequestParam @ApiParam(value = "照片地址", required = true) String photoUrl,
			@RequestParam @ApiParam(value = "操作者", required = true) String operator) {

		if (!getSecurityRootOrgId().equals(rootOrgId)) {
			throw new StatusException("EX-1000001", "rootOrgId is wrong");
		}

		if (photoUrl.startsWith("http")) {
			byte[] bs = HttpClientUtil.get(photoUrl);

			int lastIndexOf = photoUrl.lastIndexOf(".");
			if (0 > lastIndexOf) {
				throw new StatusException("EX-100002",
						"photoPath is not end with photo file suffix.");
			}
			String fileSuffix = photoUrl.substring(lastIndexOf);

			String fileName = System.currentTimeMillis() + fileSuffix;
			File temp = new File(
					tempDir + File.separator + "student_photo" + File.separator + fileName);

			try {
				FileUtils.writeByteArrayToFile(temp, bs);
			} catch (IOException e) {
				throw new StatusException("EX-100003", "文件读写失败");
			}

			faceService.processFace(rootOrgId, identityNumber, fileSuffix, temp, operator);
		}

	}

	@ApiOperation(value = "保存学生照片", httpMethod = "POST")
	@PostMapping("addPhoto")
	@Override
	public void addPhoto(
			@RequestParam @ApiParam(value = "顶级机构ID", example = "1", required = true) Long rootOrgId,
			@RequestParam @ApiParam(value = "身份证", required = true) String identityNumber,
			@RequestParam @ApiParam(value = "操作者", required = true) String operator,
			@RequestParam @ApiParam(value = "学生照片文件(文件名称=身份证号码+文件后缀) 如：xxxxxxxxxx.jpg", required = true) CommonsMultipartFile file) {

		if (!getSecurityRootOrgId().equals(rootOrgId)) {
			throw new StatusException("EX-1000001", "rootOrgId is wrong");
		}

		DiskFileItem item = (DiskFileItem) file.getFileItem();
		File storeLocation = item.getStoreLocation();
		String fileName = file.getOriginalFilename();

		if (StringUtils.containsWhitespace(fileName)) {
			throw new StatusException("EX-600100", "文件名不能含有空白字符");
		}

		if (!fileName.matches("[^\\.\\s]+\\.[^\\.\\s]+")) {
			throw new StatusException("EX-600101", "文件名不合法");
		}
		String fileSuffix = fileName.substring(fileName.lastIndexOf("."));

		faceService.processFace(rootOrgId, identityNumber, fileSuffix, storeLocation, operator);
	}

	@ApiOperation(value = "导入学生照片", httpMethod = "POST")
	@PostMapping("importPhotos")
	@Override
	public List<Map<String, String>> importPhotos(
			@RequestParam @ApiParam(value = "顶级机构ID", example = "1", required = true) Long rootOrgId,
			@RequestParam @ApiParam(value = "操作者", required = true) String operator,
			@RequestParam @ApiParam(value = "学生照片文件(身份证号码+文件后缀)压缩包(ZIP文件,压缩包内不含目录)", required = true) CommonsMultipartFile file) {

		if (!getSecurityRootOrgId().equals(rootOrgId)) {
			throw new StatusException("EX-1000001", "rootOrgId is wrong");
		}

		DiskFileItem item = (DiskFileItem) file.getFileItem();
		File storeLocation = item.getStoreLocation();
		String filename = file.getOriginalFilename();

		if (!filename.endsWith(".zip")) {
			throw new StatusException("EX-620001", "文件格式不正确，必须是zip格式的压缩包");
		}
		// 文件大小限制
		if (file.getSize() > ZIP_MAX_SIZE * 1024 * 1024) {
			throw new StatusException("EX-620002", "文件大小超过50M,请分批导入");
		}

		List<Map<String, String>> ret = Lists.newArrayList();

		String tempDirPath = tempDir + File.separator + "photo_import" + File.separator
				+ System.currentTimeMillis();
		File tempDir = new File(tempDirPath);
		try {
			ZipUtil.unzip(storeLocation, new File(tempDirPath));
		} catch (Exception e) {
			throw new StatusException("EX-620003", "zip文件损坏");
		}

		File[] files = tempDir.listFiles();

		if (null == files) {
			throw new StatusException("EX-620004", "zip文件为空");
		}

		for (File f : files) {
			Map<String, String> map = Maps.newHashMap();
			String fileName = f.getName();
			map.put("file", fileName);

			if (f.length() > MAX_SIZE * 1024) {
				map.put("statusCode", "EX-620005");
				map.put("statusDesc", "文件大小超过500KB");
				ret.add(map);
				continue;
			}
			try {
				if (StringUtils.containsWhitespace(fileName)) {
					throw new StatusException("EX-600100", "文件名不能含有空白字符");
				}

				if (!fileName.matches("[^\\.\\s]+\\.[^\\.\\s]+")) {
					throw new StatusException("EX-600101", "文件名不合法");
				}
				String identityNumber = fileName.substring(0, fileName.lastIndexOf("."));
				String fileSuffix = fileName.substring(fileName.lastIndexOf("."));

				faceService.processFace(rootOrgId, identityNumber, fileSuffix, f, operator);

				map.put("statusCode", "200");
				map.put("statusDesc", "成功");

			} catch (StatusException e) {
				map.put("statusCode", e.getCode());
				map.put("statusDesc", e.getDesc());
			} catch (Exception e) {
				map.put("statusCode", "系统异常");
				map.put("statusDesc", e.getCause().getMessage());
			}
			ret.add(map);
		}

		try {
			FileUtils.deleteDirectory(tempDir);
		} catch (IOException e) {
			log.error("fail to clean temp dir.", e);
		}

		return ret;
	}

}
