package cn.com.qmth.examcloud.exchange.outer.api.provider;

import java.util.ArrayList;
import java.util.List;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.commons.web.support.StatusResponse;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamScoreDataCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.QueryCapturePhotoBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.QueryCapturePhotoReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.QueryCapturePhotoResp;
import cn.com.qmth.examcloud.exchange.outer.api.QueryCapturePhotoService;
import cn.com.qmth.examcloud.exchange.outer.api.bean.OuterGetCapturePhotoBean;
import cn.com.qmth.examcloud.exchange.outer.api.request.OuterQueryCapturePhotoReq;
import cn.com.qmth.examcloud.exchange.outer.api.response.OuterQueryCapturePhotoResp;

@Api("获取抓拍照片信息服务")
@RestController
@RequestMapping("${$rmp.cloud.exchange.outer}/capturePhoto")
public class QueryCapturePhotoServiceProvider extends ControllerSupport implements QueryCapturePhotoService {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3051021630006959568L;
	
	@Autowired
	private ExamScoreDataCloudService examScoreDataCloudService;

	@ApiOperation(value = "获取抓拍照片信息数据", httpMethod = "POST")
	@ApiResponses({@ApiResponse(code = 200, message = "成功", response = OuterQueryCapturePhotoResp.class),
			@ApiResponse(code = 500, message = "系统异常(异常信息见响应体)", response = StatusResponse.class)})
	@PostMapping("/queryCapturePhoto")
	@Override
	public OuterQueryCapturePhotoResp queryCapturePhotoList(@RequestBody OuterQueryCapturePhotoReq req) {
		if (!getSecurityRootOrgId().equals(req.getRootOrgId())) {
			throw new StatusException("EX-1000001", "rootOrgId is wrong");
		}
		if (req.getExamRecordDataId()==null) {
			throw new StatusException("EX-1000002", "examRecordDataId is null");
		}
		QueryCapturePhotoReq queryCapturePhotoReq = new QueryCapturePhotoReq();
		queryCapturePhotoReq.setExamRecordDataId(req.getExamRecordDataId());
		QueryCapturePhotoResp capturePhotoResp = examScoreDataCloudService.queryCapturePhoto(queryCapturePhotoReq);
		OuterQueryCapturePhotoResp resp = new OuterQueryCapturePhotoResp();
		resp.setExamRecordDataId(capturePhotoResp.getExamRecordDataId());
		
		List<OuterGetCapturePhotoBean> capturePhotoList = new ArrayList<OuterGetCapturePhotoBean>();
		for(QueryCapturePhotoBean queryCapturePhotoBean:capturePhotoResp.getCapturePhotoList()){
			OuterGetCapturePhotoBean outerGetCapturePhotoBean = new OuterGetCapturePhotoBean();
			outerGetCapturePhotoBean.setFileUrl(queryCapturePhotoBean.getFileUrl());
			outerGetCapturePhotoBean.setIsPass(queryCapturePhotoBean.getIsPass());
			outerGetCapturePhotoBean.setIsStranger(queryCapturePhotoBean.getIsStranger());
			outerGetCapturePhotoBean.setIsLivenessPass(queryCapturePhotoBean.getIsLivenessPass());
			capturePhotoList.add(outerGetCapturePhotoBean);
		}
		resp.setCapturePhotoList(capturePhotoList);
		return resp;
	}

}
