package cn.com.qmth.examcloud.exchange.outer.api.provider;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.commons.web.support.StatusResponse;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamScoreObtainQueueCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.request.DeleteExamScoreQueueReq;
import cn.com.qmth.examcloud.core.oe.admin.api.request.GetTopExamScoreQueueReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.GetTopExamScoreQueueResp;
import cn.com.qmth.examcloud.exchange.outer.api.ScoreQueueService;
import cn.com.qmth.examcloud.exchange.outer.api.request.OuterDeleteScoreQueueTopReq;
import cn.com.qmth.examcloud.exchange.outer.api.request.OuterGetScoreQueueTopReq;
import cn.com.qmth.examcloud.exchange.outer.api.response.OuterDeleteScoreQueueTopResp;
import cn.com.qmth.examcloud.exchange.outer.api.response.OuterGetScoreQueueTopResp;

@Api("成绩队列服务")
@RestController
@RequestMapping("${$rmp.cloud.exchange.outer}/scoreQueue")
public class ScoreQueueServiceProvider extends ControllerSupport implements ScoreQueueService {

	/**
	 * 
	 */
	private static final long serialVersionUID = -7975687672489564748L;
	
	@Autowired
	private ExamScoreObtainQueueCloudService examScoreObtainQueueCloudService;

	@ApiOperation(value = "取队列最顶层记录", httpMethod = "POST")
	@ApiResponses({@ApiResponse(code = 200, message = "成功", response = OuterGetScoreQueueTopResp.class),
			@ApiResponse(code = 500, message = "系统异常(异常信息见响应体)", response = StatusResponse.class)})
	@PostMapping("/getScoreQueueTop")
	@Override
	public OuterGetScoreQueueTopResp getScoreQueueTop(@RequestBody OuterGetScoreQueueTopReq req) {
		if (!getSecurityRootOrgId().equals(req.getRootOrgId())) {
			throw new StatusException("EX-1000001", "rootOrgId is wrong");
		}
		GetTopExamScoreQueueReq getTopExamScoreQueueReq = new GetTopExamScoreQueueReq();
		getTopExamScoreQueueReq.setRootOrgId(req.getRootOrgId());
		GetTopExamScoreQueueResp getTopExamScoreQueueResp = examScoreObtainQueueCloudService.getTopExamScoreQueue(getTopExamScoreQueueReq);
		OuterGetScoreQueueTopResp resp = new OuterGetScoreQueueTopResp();
		if(getTopExamScoreQueueResp!=null){
			resp.setQueueId(getTopExamScoreQueueResp.getQueueId());
			resp.setExamRecordDataId(getTopExamScoreQueueResp.getExamRecordDataId());
		}
		return resp;
	}

	@ApiOperation(value = "删除队列最顶层记录", httpMethod = "POST")
	@ApiResponses({@ApiResponse(code = 200, message = "成功", response = OuterDeleteScoreQueueTopResp.class),
			@ApiResponse(code = 500, message = "系统异常(异常信息见响应体)", response = StatusResponse.class)})
	@PostMapping("/deleteScoreQueueTop")
	@Override
	public OuterDeleteScoreQueueTopResp deleteScoreQueueTop(@RequestBody OuterDeleteScoreQueueTopReq req) {
		if (!getSecurityRootOrgId().equals(req.getRootOrgId())) {
			throw new StatusException("EX-1000001", "rootOrgId is wrong");
		}
		GetTopExamScoreQueueReq getTopExamScoreQueueReq = new GetTopExamScoreQueueReq();
		getTopExamScoreQueueReq.setRootOrgId(req.getRootOrgId());
		GetTopExamScoreQueueResp getTopExamScoreQueueResp = examScoreObtainQueueCloudService.getTopExamScoreQueue(getTopExamScoreQueueReq);
		
		if(!getTopExamScoreQueueResp.getQueueId().equals(req.getQueueId())){
			throw new StatusException("EX-1000003", "queueId is wrong");
		}
		
		DeleteExamScoreQueueReq deleteExamScoreQueueReq = new DeleteExamScoreQueueReq();
		deleteExamScoreQueueReq.setQueueId(req.getQueueId());
		deleteExamScoreQueueReq.setRootOrgId(req.getRootOrgId());
		examScoreObtainQueueCloudService.deleteExamScoreQueue(deleteExamScoreQueueReq);
		OuterDeleteScoreQueueTopResp resp = new OuterDeleteScoreQueueTopResp();
		resp.setIsSuccess(true);
		resp.setDesc("删除成功");
		return resp;
	}

}
